@@ -7,6 +7,7 @@ import datetime
 import uuid
 import base64
 import os
+from datetime import datetime
 from .dynamo_db import DynamoDb
 from .run_status import RunStatus
 from .fsx import Fsx
@@ -22,12 +23,12 @@ class ContainerLauncher:
     __ASG_FINISHING_SCALING_ACTIVITIES_TIMEOUT_IN_MINUTES = 5
     __EC2_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES = 5
     __EC2_INSTANCE_REGISTRATION_TIMEOUT_IN_MINUTES = 5
+    __EC2_INSTANCE_INITIALIZING_TIMEOUT_IN_MINUTES = 20
     __EC2_INSTANCE_UNUSED_MEMORY = 512
     __ECS_TASK_LOG_STREAM_PREFIX = 'acl-task'
     __ECS_TASK_EXECUTION_POLICY_ARN = 'arn:aws:iam::aws:policy/service-role/AmazonECSTaskExecutionRolePolicy'
     __IAM_EC2_MANAGED_POLICY_ARNS = [
-        'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceforEC2Role',
-        # 'arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCore'
+        'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceforEC2Role'
     ]
 
     def __init__(self, options=None):
@@ -277,6 +278,20 @@ class ContainerLauncher:
         run_status = entry['status']
 
         if run_status != RunStatus.IN_PROGRESS.name:
+            if run_status == RunStatus.INITIALIZING.name:
+                create_timestamp = entry['create_timestamp']
+                diff_in_minutes = self.__get__get_difference_in_minutes_with_now(create_timestamp)
+                if diff_in_minutes > self.__EC2_INSTANCE_INITIALIZING_TIMEOUT_IN_MINUTES:
+                    code = 'InstanceInitializingTimeoutException'
+                    self.__dynamodb.update_status(entry_id, RunStatus.FAILED.name)
+                    self.__dynamodb.add_reason(entry_id, code)
+                    return {
+                        'status': RunStatus.FAILED.name,
+                        'error': {
+                            'code': code
+                        }
+                    }
+
             response = {
                 'status': run_status
             }
@@ -775,11 +790,11 @@ class ContainerLauncher:
     def __wait_for_draining_asg(self, asg_name):
         response = self.clients['autoscaling'].describe_auto_scaling_groups(AutoScalingGroupNames=[asg_name])
 
-        start_time = datetime.datetime.now()
+        start_time = datetime.now()
 
         while len(response['AutoScalingGroups'][0]['Instances']) != 0:
             self.logger.debug('Waiting for draining ASG')
-            now_time = datetime.datetime.now()
+            now_time = datetime.now()
             diff = now_time - start_time
             diff_minutes = diff.total_seconds() / 60
             if diff_minutes > self.__ASG_DRAINING_TIMEOUT_IN_MINUTES:
@@ -804,7 +819,7 @@ class ContainerLauncher:
 
         instances = [] if len(response['Reservations']) == 0 else response['Reservations'][0]['Instances']
 
-        start_time = datetime.datetime.now()
+        start_time = datetime.now()
 
         if len(instances) == 0:
             self.logger.info('Nothing to terminate. EC2 instance already terminated')
@@ -813,7 +828,7 @@ class ContainerLauncher:
         self.logger.debug(f'Waiting for EC2 instance termination')
 
         while instances[0]['State']['Name'] != 'terminated':
-            now_time = datetime.datetime.now()
+            now_time = datetime.now()
             diff = now_time - start_time
             diff_minutes = diff.total_seconds() / 60
             if diff_minutes > self.__INSTANCE_TERMINATION_TIMEOUT_IN_MINUTES:
@@ -839,10 +854,10 @@ class ContainerLauncher:
     def __wait_for_finishing_scaling_activities(self, asg_name):
         activities = self.__get_not_final_asg_activities(asg_name)
 
-        start_time = datetime.datetime.now()
+        start_time = datetime.now()
         while len(activities) != 0:
             self.logger.debug(f'Waiting for finishing ASG activities. Num: {len(activities)}')
-            now_time = datetime.datetime.now()
+            now_time = datetime.now()
             diff = now_time - start_time
             diff_minutes = diff.total_seconds() / 60
             if diff_minutes > self.__ASG_FINISHING_SCALING_ACTIVITIES_TIMEOUT_IN_MINUTES:
@@ -862,10 +877,10 @@ class ContainerLauncher:
         attachments_status = \
             self.clients['ecs'].describe_clusters(clusters=[ecs_cluster_name], include=['ATTACHMENTS'])['clusters'][0][
                 'attachmentsStatus']
-        start_time = datetime.datetime.now()
+        start_time = datetime.now()
         while attachments_status == 'UPDATE_IN_PROGRESS':
             self.logger.info(f'Waiting for updating the ECS cluster attachments')
-            now_time = datetime.datetime.now()
+            now_time = datetime.now()
             diff = now_time - start_time
             diff_minutes = diff.total_seconds() / 60
             if diff_minutes > self.__ECS_CLUSTER_ATTACHMENTS_UPDATE_TIMEOUT_IN_MINUTES:
@@ -1025,14 +1040,14 @@ class ContainerLauncher:
     def __wait_for_instance_deploying(self, is_spot):
         instances = self.__get_asg_instances()
 
-        start_time = datetime.datetime.now()
+        start_time = datetime.now()
 
         timeout_in_minutes = self.__SPOT_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES if is_spot else self.__EC2_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES
 
         self.logger.info(f'Waiting for an EC2 instance deploying ({timeout_in_minutes} minutes)')
 
         while True:
-            now_time = datetime.datetime.now()
+            now_time = datetime.now()
             diff = now_time - start_time
             diff_minutes = diff.total_seconds() / 60
             if diff_minutes > timeout_in_minutes:
@@ -1062,11 +1077,11 @@ class ContainerLauncher:
         cluster_name = self.__resources['ecs_cluster_name']
         registered_instances_count = self.__get_ecs_cluster_registered_instances_count(cluster_name)
 
-        start_time = datetime.datetime.now()
+        start_time = datetime.now()
 
         while True:
             self.logger.debug('Waiting for an EC2 instance registration in ECS')
-            now_time = datetime.datetime.now()
+            now_time = datetime.now()
             diff = now_time - start_time
             diff_minutes = diff.total_seconds() / 60
             if diff_minutes > self.__EC2_INSTANCE_REGISTRATION_TIMEOUT_IN_MINUTES:
@@ -1293,6 +1308,14 @@ mount /dev/md0 %s
 
         return ''
 
+    @staticmethod
+    def __get__get_difference_in_minutes_with_now(date_string):
+        start_date = datetime.strptime(date_string, '%Y-%m-%d %H:%M:%S.%f')
+        now = datetime.now()
+        diff = now - start_date
+        minutes = diff.total_seconds() / 60
+        return minutes
+
     @staticmethod
     def __filter_not_terminated_instances(ec2_instance):
         if ec2_instance.get('State') is not None and ec2_instance['State']['Name'] == 'terminated':
